@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2017  Ruby-GNOME2 Project Team
+# Copyright (C) 2005-2021  Ruby-GNOME Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -107,40 +107,56 @@ module GLib
     begin
       require "ruby_installer/runtime"
     rescue LoadError
-      prepend_path_to_environment_variable(path, "PATH")
     else
       RubyInstaller::Runtime.add_dll_directory(path.to_s)
     end
+    prepend_path_to_environment_variable(path, "PATH")
   end
 end
 
-
-base_dir = Pathname.new(__FILE__).dirname.dirname.expand_path
-vendor_dir = base_dir + "vendor" + "local"
-if vendor_dir.exist?
-  require "cairo"
-end
-
-GLib.prepend_dll_path(vendor_dir + "bin")
-begin
-  major, minor, _ = RUBY_VERSION.split(/\./)
-  require "#{major}.#{minor}/glib2.so"
-rescue LoadError
-  require 'glib2.so'
-end
+require "glib2.so"
 
 module GLib
+  SIGNAL_HANDLER_PREFIX = "signal_do_"
+  VIRTUAL_FUNCTION_IMPLEMENTATION_PREFIX = "virtual_do_"
+
   module MetaInterface
     class << self
-      def signal_callback(klass, id)
+      def signal_callback(klass, name)
         lambda do |instance, *args|
-          klass.instance_method(id).bind(instance).call(*args)
+          method_name = "#{SIGNAL_HANDLER_PREFIX}#{name}"
+          klass.instance_method(method_name).bind(instance).call(*args)
         end
       end
     end
   end
 
   class Instantiatable
+    class << self
+      def method_added(name)
+        super
+
+        case name.to_s
+        when /\A#{Regexp.escape(SIGNAL_HANDLER_PREFIX)}/o
+          signal_name = $POSTMATCH
+          begin
+            signal_ = signal(signal_name)
+          rescue NoSignalError
+            return
+          end
+          return unless signal_.class != self
+          signal_handler_attach(signal_, name.to_s) do |instance, *args|
+            instance.__send__(name, *args)
+          end
+        when /\A#{Regexp.escape(VIRTUAL_FUNCTION_IMPLEMENTATION_PREFIX)}/o
+          ancestors.each do |klass|
+            next unless klass.respond_to?(:implement_virtual_function)
+            return if klass.implement_virtual_function(self, name)
+          end
+        end
+      end
+    end
+
     private
     def create_signal_handler(signal_name, callback)
       callback
@@ -161,23 +177,50 @@ module GLib
 
 
   class Enum
-    def _dump(limit)
-      Marshal.dump(to_i, limit)
+    class << self
+      def try_convert(value)
+        if value.is_a?(self)
+          value
+        else
+          find(value)
+        end
+      end
+
+      def _load(obj)
+        new(Marshal.load(obj))
+      end
     end
 
-    def self._load(obj)
-      new(Marshal.load(obj))
+    def _dump(limit)
+      Marshal.dump(to_i, limit)
     end
   end
 
 
   class Flags
-    def _dump(limit)
-      Marshal.dump(to_i, limit)
+    class << self
+      def try_convert(value)
+        case value
+        when self
+          value
+        when Integer, String, Symbol, Array
+          begin
+            new(value)
+          rescue ArgumentError
+            nil
+          end
+        else
+          nil
+        end
+      end
+
+      def _load(obj)
+        new(Marshal.load(obj))
+      end
     end
 
-    def self._load(obj)
-      new(Marshal.load(obj))
+    def _dump(limit)
+      Marshal.dump(to_i, limit)
     end
 
     # FIXME
@@ -269,8 +312,9 @@ GLib::Log.set_log_domain(GLib::Object::LOG_DOMAIN)
 GLib::Log.set_log_domain(GLib::Thread::LOG_DOMAIN)
 GLib::Log.set_log_domain(GLib::Module::LOG_DOMAIN)
 
-require 'glib2/version'
+require "glib2/version"
 require "glib2/regex"
+require "glib2/deprecated"
 =begin
 Don't we need this?
 ObjectSpace.define_finalizer(GLib) {
